red <- "<font color=\"red\">"
np <- 100
x <- c(0.5,rep(seq(0,1,length=np),np))
y <- c(0.5,rep(seq(0,1,length=np),each=np))
xy <- data.frame(x=x,y=y)

shinyServer(function(input, output) {  

    gotLibs <- reactive({
        a<-is.element("ggplot2", installed.packages()[,1])
        b<-is.element("grid", installed.packages()[,1])
        x <- c(a,b,a&b)
        names(x) <- c("ggplot2","grid","both")
        x
    })
    output$checkGraph <- renderText({
        if(input$whichgraph=="base") return("")
        if(gotLibs()[1]) library(ggplot2)
#        if(gotLibs()[2]) library(grid)    
        if(!gotLibs()[3]) return(paste(red,"<h4>Warning: some R libraries not found, will use base graphics</h4>"))
        NULL       
    })    

   vals <- reactiveValues(
    keeprows = c(FALSE,rep(FALSE, np*np))
  )


  output$textpoints <- renderText({

    keep    <- xy[!vals$keeprows, , drop = FALSE]
    exclude <- xy[vals$keeprows, , drop = FALSE]
    
    if(nrow(exclude)<=1) return("<h5>We need at least two points!</h5>")
    line <- paste("<h5>","Correlation:",round(cor(exclude)[1,2],3),"</h5>")
    
    if(input$reg1=="no") return(line)
    co <- round(coef(lm(exclude[,2]~exclude[,1])),3)
    sn <- ifelse(co[2]>0,"+","-")
    line[2] <- paste("<h5>Least Squares Regression Line:</h5>")
    line[3] <- paste("<h5>Y=",co[1],sn,abs(co[2]),"x</h5>")
    line
  })

  output$plotpoints <- renderPlot({

    keep    <- xy[!vals$keeprows, , drop = FALSE]
    exclude <- xy[vals$keeprows, , drop = FALSE]
    
    if(input$whichgraph=="base" | !gotLibs()[1]) {
        plot(xy,type="n",xlab="",ylab="")
        points(exclude,pch=20,cex=1.3)
        if(nrow(exclude)>1) 
          if(input$reg1=="yes") 
              abline(lm(exclude[,2]~exclude[,1]),lwd=2)
    }
    else {
        plt <- ggplot(xy, aes(x, y)) +
              geom_point(data = exclude, shape = 20, fill = NA, color = "black", alpha = 1,size=3) +
              coord_cartesian(xlim = c(0, 1), ylim = c(0,1))+
              xlab("")+ylab("")    
         if(nrow(exclude)>1 & input$reg1=="yes")
             plt <- plt+ stat_smooth(data=exclude,method = "lm",se=FALSE,size=1.2)
         print(plt)     
    }   
  })

  observeEvent(input$plot1_click, {
    res <- nearPoints(xy, input$plot1_click, threshold = 5, 
          xvar="x",yvar="y",maxpoints = 1, allRows = TRUE)

    vals$keeprows <- xor(vals$keeprows, res$selected_)
  })


  # Reset all points
  observeEvent(input$exclude_reset, {
    vals$keeprows <- c(FALSE,rep(FALSE, np*np))
  })
      
  data <- reactive({
      x <- rnorm(input$n) 
      if(abs(input$rho)==1) y <- sign(input$rho)*x
      else {
        a <- input$rho/sqrt(1-input$rho^2)
        y <- a*x+rnorm(input$n)
      }    
      if(input$out=="Yes") {
        x[1:input$m] <- runif(input$m,-3,-2)
        y[1:input$m] <- runif(input$m,5,7)
      }    
      cbind(x,y)
  })
        
  output$scplot <- renderPlot({
    x <- data()[,1]
    y <- data()[,2]
    if(input$whichgraph=="base") {
        plot(x,y,pch=20,xlab="x",ylab="y")
        if(input$out=="Yes") 
            points(x[1:input$m],y[1:input$m],pch=20,cex=2,col="red")
        if(input$reg2=="yes") 
              abline(lm(y~x),lwd=2)    
    }
    else {
        if(is.element("ggplot2", installed.packages()[,1])) {
            dta <- data.frame(x=x,y=y,Outlier=rep("No",length(x)))
            if(input$out=="Yes") {
                dta$Outlier[1:input$m]="Yes"
                plt <- ggplot(aes(x,y,colour=Outlier),data=dta)+
                       geom_point(size=2)
            }
            else  plt <- ggplot(aes(x,y),data=dta)+
                        geom_point(size=2)
            if(input$reg2=="yes")
                print(plt+ stat_smooth(data=dta,method = "lm",se=FALSE,size=1.2))
            else print(plt)     
        }
    }        
  })
    
  data1 <- reactive({
        if(abs(input$rho)==1) return(NULL)
        a <- input$rho/sqrt(1-input$rho^2)
        r <- rep(0,1000)
        for(i in 1:1000) {
           x <- rnorm(input$n) 
           y <- a*x+rnorm(input$n)
           r[i] <- cor(x,y)
        }
        r
  })   
     
  output$histo <- renderPlot({  
      x <- data1()
      if(input$whichgraph=="base") {
        hist(x,breaks=100,xlim=c(-1,1),xlab="r",ylab="",main="")              
        abline(v=0,lwd=3)
      }
      else {
        if(is.element("ggplot2", installed.packages()[,1])) {
            x <- data.frame(x=x)
            ggplot(x, aes(x=x))+
                geom_histogram(binwidth=0.02,color="black",fill="white")+
                expand_limits(x=c(-1,1))
                
        }      
      }  
  })    

  output$text3 <- renderText({  
        line <- paste("<h3>Sample correlation coefficients for case &rho; =",input$rho," and n =",input$n,"<h3>")
        line[2] <-""
        if(input$rho<0)
            line[2] <- paste("<h3>Percentage of r<0: ",round(sum(data1()<0)/10,1),"%</h3>")
        else
            line[2] <- paste("<h3>Percentage of r>0: ",round(sum(data1()>0)/10,1),"%</h3>")    
        line    
  })    
  
  output$text1 <- renderText({  
        pc <- paste(sprintf("%0.3f",input$rho))
        line <- paste("<h5>&nbsp;&nbsp;&nbsp;&nbsp;Parameter Correlation Coefficient: ",HTML("&rho;")," = ",pc,"<h5>",collapse="")
        sc <- paste(sprintf("%0.3f",cor(data())[1,2]))
        line[2] <- paste("<h5>&nbsp;&nbsp;&nbsp;&nbsp;Sample Correlation Coefficient r = ",sc,"</h5>",collapse="")
        if(input$reg2=="no") return(line)
        co <- round(coef(lm(data()[,2]~data()[,1])),3)
        sn <- ifelse(co[2]>0,"+","-")
        line[3] <- paste("<h5>&nbsp;&nbsp;&nbsp;&nbsp;Least Squares Regression Line: 
                          Y =",co[1],sn,abs(co[2]),"x</h5>")
        line
  })
  
  output$text2 <- renderText({
        pval <- cor.test(data()[,1],data()[,2])$p.value
        sgn <- ifelse(pval>0.05," &gt; "," &lt; ")
        rej <- ifelse(pval>0.05," fail to reject "," reject ")
        dec <- ifelse(pval>0.05," X and Y are uncorreleated "
                    ," X and Y are correlated ")
        txt <- "<h5>1) Parameter: Pearson's Correlation Coefficient &rho;</h5>
                <h5>2) Method: Standard Test</h5>"
        txt[2] <- "<h5>3) Assumption: Data comes from Normal Distribution</h5>"
        if(input$out=="No") 
            txt[3] <- "<h5>Assumption is ok</h5>"
        else txt[3] <- "<h5>Assumption is NOT ok</h5>"    
        txt[4] <- "<h5>4) &alpha;  = 0.05</h5>"
        txt[5] <- "<h5>5) H<sub>0</sub>: &rho;  = 0</h5>"
        txt[6] <- "<h5>5) H<sub>0</sub>: &rho;  &ne; 0</h5>" 
        txt[7] <- paste("<h5>7) p =",round(pval,4),"</h5>",sep="")        
        txt[8] <- paste("<h5>8) ",round(pval,4),sgn," &alpha;, so
                  we",rej," the null hypothesis, ",dec,"</h5>",sep="")
        txt            
  })      
 
  output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.htm#correlation\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 11/22/2014, last updated 10/5/2015</h6>")     
  })   
  
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates Pearsons's Correlation Coefficient </h5>"
        
  })
})

